/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.generator;

import net.minecraft.class_1538;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2902;
import net.minecraft.class_3218;
import reborncore.api.IToolDrop;
import reborncore.common.blocks.BlockMachineBase;
import reborncore.common.powerSystem.PowerAcceptorBlockEntity;
import reborncore.common.registration.RebornRegister;
import reborncore.common.registration.config.ConfigRegistry;
import techreborn.TechReborn;
import techreborn.init.TRContent;
import techreborn.init.TRBlockEntities;

@RebornRegister(TechReborn.MOD_ID)
public class LightningRodBlockEntity extends PowerAcceptorBlockEntity implements IToolDrop {

	@ConfigRegistry(config = "generators", category = "lightning_rod", key = "LightningRodMaxOutput", comment = "Lightning Rod Max Output (Value in EU)")
	public static int maxOutput = 2048;
	@ConfigRegistry(config = "generators", category = "lightning_rod", key = "LightningRodMaxEnergy", comment = "Lightning Rod Max Energy (Value in EU)")
	public static int maxEnergy = 100_000_000;
	@ConfigRegistry(config = "generators", category = "lightning_rod", key = "LightningRodChanceOfStrike", comment = "Chance of lightning striking a rod (Range: 0-70)")
	public static int chanceOfStrike = 24;
	@ConfigRegistry(config = "generators", category = "lightning_rod", key = "LightningRodBaseStrikeEnergy", comment = "Base amount of energy per strike (Value in EU)")
	public static int baseEnergyStrike = 262_144;

	private int onStatusHoldTicks = -1;

	public LightningRodBlockEntity() {
		super(TRBlockEntities.LIGHTNING_ROD);
	}

	@Override
	public void method_16896() {
		super.method_16896();

		if (onStatusHoldTicks > 0) { --onStatusHoldTicks; }

		class_2248 BEBlock = method_11010().method_11614();
		if (! (BEBlock instanceof BlockMachineBase)) {
			return;
		}
		
		BlockMachineBase machineBaseBlock = (BlockMachineBase) BEBlock;
		
		if (onStatusHoldTicks == 0 || getEnergy() <= 0) {
			machineBaseBlock.setActive(false, field_11863, field_11867);
			onStatusHoldTicks = -1;
		}

		final float weatherStrength = field_11863.method_8478(1.0F);
		if (weatherStrength > 0.2F) {
			//lightStrikeChance = (MAX - (CHANCE * WEATHER_STRENGTH)
			final float lightStrikeChance = (100F - chanceOfStrike) * 20F;
			final float totalChance = lightStrikeChance * getLightningStrikeMultiplier() * (1.1F - weatherStrength);
			if (field_11863.field_9229.nextInt((int) Math.floor(totalChance)) == 0) {
				if (!isValidIronFence(field_11867.method_10084().method_10264())) {
					onStatusHoldTicks = 400;
					return;
				}
				final class_1538 lightningBolt = new class_1538(field_11863,
					field_11867.method_10263() + 0.5F, field_11863.method_8598(class_2902.class_2903.field_13197, method_11016()).method_10264(),
					field_11867.method_10260() + 0.5F, false);
				
				if (!field_11863.field_9236) {
					((class_3218) field_11863).method_8416(lightningBolt);
				}
				addEnergy(baseEnergyStrike * (0.3F + weatherStrength));
				machineBaseBlock.setActive(true, field_11863, field_11867);
				onStatusHoldTicks = 400;
			}
		}

	}

	public float getLightningStrikeMultiplier() {
		final float actualHeight = 256;
		final float groundLevel = field_11863.method_8598(class_2902.class_2903.field_13197, method_11016()).method_10264();
		for (int i = field_11867.method_10264() + 1; i < actualHeight; i++) {
			if (!isValidIronFence(i)) {
				if (groundLevel >= i)
					return 4.3F;
				final float max = actualHeight - groundLevel;
				final float got = i - groundLevel;
				return 1.2F - got / max;
			}
		}
		return 4F;
	}

	public boolean isValidIronFence(int y) {
		class_2248 block = this.field_11863.method_8320(new class_2338(field_11867.method_10263(), y, field_11867.method_10260())).method_11614();
		if(block == TRContent.REFINED_IRON_FENCE){
			return true;
		}
		return false;
	}

	@Override
	public double getBaseMaxPower() {
		return maxEnergy;
	}

	@Override
	public boolean canAcceptEnergy(final class_2350 direction) {
		return false;
	}

	@Override
	public boolean canProvideEnergy(final class_2350 direction) {
		return true;
	}

	@Override
	public double getBaseMaxOutput() {
		return maxOutput;
	}

	@Override
	public double getBaseMaxInput() {
		return 0;
	}

	@Override
	public class_1799 getToolDrop(class_1657 playerIn) {
		return TRContent.Machine.LIGHTNING_ROD.getStack();
	}
}
