/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.storage.lesu;

import reborncore.api.power.EnumPowerTier;
import reborncore.client.containerBuilder.IContainerProvider;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.containerBuilder.builder.ContainerBuilder;
import reborncore.common.RebornCoreConfig;
import reborncore.common.registration.RebornRegister;
import reborncore.common.registration.config.ConfigRegistry;
import techreborn.TechReborn;
import techreborn.blocks.storage.BlockLapotronicSU;
import techreborn.init.TRContent;
import techreborn.init.TRBlockEntities;
import techreborn.blockentity.storage.EnergyStorageBlockEntity;

import java.util.ArrayList;
import net.minecraft.class_1657;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2586;

@RebornRegister(TechReborn.MOD_ID)
public class LapotronicSUBlockEntity extends EnergyStorageBlockEntity implements IContainerProvider{

//	@ConfigRegistry(config = "machines", category = "lesu", key = "LesuMaxInput", comment = "LESU Max Input (Value in EU)")
//	public static int maxInput = 8192;
	@ConfigRegistry(config = "machines", category = "lesu", key = "LesuMaxOutput", comment = "LESU Base Output (Value in EU)")
	public static int baseOutput = 16;
	@ConfigRegistry(config = "machines", category = "lesu", key = "LesuMaxEnergyPerBlock", comment = "LESU Max Energy Per Block (Value in EU)")
	public static int storagePerBlock = 1_000_000;
	@ConfigRegistry(config = "machines", category = "lesu", key = "LesuExtraIO", comment = "LESU Extra I/O Multiplier")
	public static int extraIOPerBlock = 8;

	public int connectedBlocks = 0;
	private ArrayList<LesuNetwork> countedNetworks = new ArrayList<>();

	public LapotronicSUBlockEntity() {
		super(TRBlockEntities.LAPOTRONIC_SU, "LESU", 2, TRContent.Machine.LAPOTRONIC_SU.block, EnumPowerTier.INSANE, 8192, baseOutput, 1_000_000);
		checkOverfill = false;
	}

	@Override
	public void method_16896() {
		super.method_16896();
		if (field_11863.field_9236) {
			return;
		}
		countedNetworks.clear();
		connectedBlocks = 0;
		for (class_2350 dir : class_2350.values()) {
			class_2338 adjucentBlockPos = new class_2338(field_11867.method_10263() + dir.method_10148(),
					field_11867.method_10264() + dir.method_10148(), field_11867.method_10260() + dir.method_10148());
			class_2586 adjucent = field_11863.method_8321(adjucentBlockPos);
			if (adjucent == null || !(adjucent instanceof LSUStorageBlockEntity)) {
				continue;
			}
			if (((LSUStorageBlockEntity) adjucent).network == null) {
				continue;
			}
			LesuNetwork network = ((LSUStorageBlockEntity) adjucent).network;
			if (!countedNetworks.contains(network)) {
				if (network.master == null || network.master == this) {
					connectedBlocks += network.storages.size();
					countedNetworks.add(network);
					network.master = this;
					break;
				}
			}

		}
		setMaxStorage();
		maxOutput = (connectedBlocks * extraIOPerBlock) + baseOutput;
	}

	@Override
	public class_2350 getFacingEnum() {
		class_2248 block = field_11863.method_8320(field_11867).method_11614();
		if (block instanceof BlockLapotronicSU) {
			return ((BlockLapotronicSU) block).getFacing(field_11863.method_8320(field_11867));
		}
		return null;
	}
	
	public int getOutputRate() {
		return maxOutput;
	}
	
	public void setOutputRate(int output) {
		this.maxOutput = output;
	}
	
	public int getConnectedBlocksNum() {
		return connectedBlocks;
	}
	
	public void setConnectedBlocksNum(int value) {
		this.connectedBlocks = value;
		if (field_11863.field_9236) {
			setMaxStorage();
		}
	}
	
	public void setMaxStorage(){
		maxStorage  = (connectedBlocks + 1) * storagePerBlock;
		if (maxStorage < 0 || maxStorage > Integer.MAX_VALUE / RebornCoreConfig.euPerFU) {
			maxStorage = Integer.MAX_VALUE / RebornCoreConfig.euPerFU;
		}
	}

	@Override
	public BuiltContainer createContainer(int syncID, final class_1657 player) {
		return new ContainerBuilder("lesu").player(player.field_7514).inventory().hotbar().armor().complete(8, 18)
				.addArmor().addInventory().blockEntity(this).energySlot(0, 62, 45).energySlot(1, 98, 45).syncEnergyValue()
				.syncIntegerValue(this::getOutputRate, this::setOutputRate)
				.syncIntegerValue(this::getConnectedBlocksNum, this::setConnectedBlocksNum).addInventory().create(this, syncID);
	}
}
